Package: fdrtool
Version: 1.2.18
Date: 2024-08-20
Title: Estimation of (Local) False Discovery Rates and Higher Criticism
Authors@R: c(person(given = "Bernd",
                      family = "Klaus",
                      role = "aut"),
             person(given = "Korbinian",
                      family = "Strimmer",
                      role = c("aut", "cre"),
                      email = "strimmerlab@gmail.com"))
Depends: R (>= 3.4.0)
Suggests:
Imports: graphics, grDevices, stats
Description: Estimates both tail area-based false 
   discovery rates (Fdr) as well as local false discovery rates (fdr) for a 
   variety of null models (p-values, z-scores, correlation coefficients,
   t-scores).  The proportion of null values and the parameters of the null 
   distribution are adaptively estimated from the data.  In addition, the package 
   contains functions for non-parametric density estimation (Grenander estimator), 
   for monotone regression (isotonic regression and antitonic regression with weights),
   for computing the greatest convex minorant (GCM) and the least concave majorant (LCM), 
   for the half-normal and correlation distributions, and for computing
   empirical higher criticism (HC) scores and the corresponding decision threshold.
License: GPL (>= 3)
URL: https://strimmerlab.github.io/software/fdrtool/
NeedsCompilation: yes
Packaged: 2024-08-20 09:34:07 UTC; strimmer
Author: Bernd Klaus [aut],
  Korbinian Strimmer [aut, cre]
Maintainer: Korbinian Strimmer <strimmerlab@gmail.com>
Repository: CRAN
Date/Publication: 2024-08-20 10:40:25 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:06:23 UTC; unix
Archs: fdrtool.so.dSYM

Package: fdrtool
Version: 1.2.18
Date: 2024-08-20
Title: Estimation of (Local) False Discovery Rates and Higher Criticism
Authors@R: c(person(given = "Bernd",
                      family = "Klaus",
                      role = "aut"),
             person(given = "Korbinian",
                      family = "Strimmer",
                      role = c("aut", "cre"),
                      email = "strimmerlab@gmail.com"))
Depends: R (>= 3.4.0)
Suggests:
Imports: graphics, grDevices, stats
Description: Estimates both tail area-based false 
   discovery rates (Fdr) as well as local false discovery rates (fdr) for a 
   variety of null models (p-values, z-scores, correlation coefficients,
   t-scores).  The proportion of null values and the parameters of the null 
   distribution are adaptively estimated from the data.  In addition, the package 
   contains functions for non-parametric density estimation (Grenander estimator), 
   for monotone regression (isotonic regression and antitonic regression with weights),
   for computing the greatest convex minorant (GCM) and the least concave majorant (LCM), 
   for the half-normal and correlation distributions, and for computing
   empirical higher criticism (HC) scores and the corresponding decision threshold.
License: GPL (>= 3)
URL: https://strimmerlab.github.io/software/fdrtool/
NeedsCompilation: yes
Packaged: 2024-08-20 09:34:07 UTC; strimmer
Author: Bernd Klaus [aut],
  Korbinian Strimmer [aut, cre]
Maintainer: Korbinian Strimmer <strimmerlab@gmail.com>
Repository: CRAN
Date/Publication: 2024-08-20 10:40:25 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:56:39 UTC; unix
Archs: fdrtool.so.dSYM

export(
"censored.fit",
"dcor0",	
"dhalfnorm", 	
"fdrtool",	
"fitted.monoreg",
"fndr.cutoff",	
"gcmlcm", 	
"grenander",
"hc.score",
"hc.thresh", 		
"monoreg", 	
"pcor0", 
"phalfnorm", 	
"plot.grenander", 
"plot.monoreg", 	
"pval.estimate.eta0", 		
"qcor0",	
"qhalfnorm", 	
"rcor0", 	
"residuals.monoreg", 	
"rhalfnorm",	
"sd2theta",	
"theta2sd"
)
S3method("fitted", "monoreg")
S3method("plot", "grenander")
S3method("plot", "monoreg")
S3method("residuals", "monoreg")

import("graphics")
import("grDevices")
import("stats")

useDynLib("fdrtool", .registration = TRUE)

### fndr.cutoff.R  (2008-03-05)
###
###     Find Approximate Cutoff Point by an FNDR Criterion
###
### Copyright 2007-08 Korbinian Strimmer 
###
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA


# find approximate cutoff based on false nondiscovery rate
fndr.cutoff <- function(x, statistic=c("normal", "correlation", "pvalue", "studentt"))
{
  statistic <- match.arg(statistic)
  nm = get.nullmodel(statistic)

  if(statistic=="pvalue")
  {
    ax = 1-x
    F0 <- function(zeta) return( nm$F0(zeta, sc.param) )
  }
  else
  {
    ax = abs(x)
    F0 <- function(zeta) return( 2*nm$F0(zeta, sc.param)-1 )
  }
 
  fndrfunc = function(x)
  {
    F.x = sum(ax < x)/length(ax)
    
    if (F.x == 0)
    { 
      FNDR.x = 0
    }
    else
    {
      FNDR.x = max(0, (F.x-e0.guess*F0(x)) / F.x)
    }

    return(FNDR.x)
  }


  # first, find approximate null model ("guess")
  g  = approximate.fit(x, statistic)
  sc.param = g$param
  e0.guess = g$eta0

  # second, find cutoff such that fndr is as small as possible
  MAXPCT0 = 0.99 # never use all the data
  zeta0 = as.double(quantile(ax, probs=min(MAXPCT0, e0.guess)))
  #cat("DEBUG: zeta0 =", zeta0, "\n")

  fndr2 = fndrfunc(zeta0)
  fndr1 = fndrfunc(0.9*zeta0)
  while ( fndr1 < fndr2 )
  {
    zeta0 = zeta0 *0.9
    #cat("DEBUG: zeta0 =", zeta0, "\n")
    fndr2 = fndr1
    fndr1 = fndrfunc(0.9*zeta0)
  }

  if(statistic == "pvalue") x0 = 1-zeta0
  else x0 = zeta0 
  names(x0) = "cutoff"

  rm(ax)

  return(x0)
}

### nullmodel.R  (2009-11-19)
###
###     Details on the FDR Null Model
###
### Copyright 2007-2009 Korbinian Strimmer 
###
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA



# this function specifies all that is needed to define a null model
get.nullmodel = function(
  statistic=c("normal", "correlation", "pvalue", "studentt")
)
{
  statistic <- match.arg(statistic)

  ###### normal z-scores ######

  if (statistic=="normal")
  {
    # untruncated density
    f0 = function(x, param, log=FALSE)
    {
      return( dnorm(x, sd=param, log=log) )
    }

    # corresponding distribution function 
    F0 = function(x, param)
    {
      return( pnorm(x, sd=param) )
    }

    # interquartile range
    iqr = function(param)
    {
       return( qnorm(.75, sd=param)-qnorm(.25, sd=param) )
    }

    # parameter support
    get.support = function() return( c(1e-9,Inf) )
  }

  ###### p-values ######

  if (statistic=="pvalue")
  {
    # untruncated density
    f0 = function(x, param, log=FALSE)
    {
      if(log==TRUE) 
        return( rep(0, length(x)) )
      else 
        return( rep(1, length(x)) )
    }

    # corresponding distribution function 
    F0 = function(x, param)
    {
      return( x )
    }

    # interquartile range
    iqr = function(param)
    {
       return( 0.5 )
    }

    # parameter support
    get.support = NULL
  }

  ###### correlation coefficients ######

  if (statistic=="correlation")
  {
    # untruncated density
    f0 = function(x, param, log=FALSE)
    {
       return( dcor0(x, kappa=param, log=log) ) 
    }

    # corresponding distribution function 
    F0 = function(x, param)
    {
      return( pcor0(x, kappa=param) )
    }

    # interquartile range
    iqr = function(param)
    {
       return( qcor0(.75, kappa=param)-qcor0(.25, kappa=param) )
    }

    # parameter support
    get.support = function() return( c(3,1e9) )
  }

  ###### t scores ######

  if (statistic=="studentt")
  {
    # untruncated density
    f0 = function(x, param, log=FALSE)
    {
       return( dt(x, df=param, log=log) )
    }

    # corresponding distribution function 
    F0 = function(x, param)
    {
      return( pt(x, df=param) )
    }

    # interquartile range
    iqr = function(param)
    {
       return( qt(.75, df=param)-qt(.25, df=param) )
    }

    # parameter support
    get.support = function() return( c(1,1000) )
  }

  ###### corresponding p-values ######

  if (statistic=="pvalue")
  {
    # one-sided p-values
    get.pval = function(x, param)
    {
      #return( F0(x) )
      return( x )  
    }
  }
  else
  {
    # two-sided p-values
    get.pval = function(x, param)
    {
      ax = abs(x)

      # return pval=0 only if abs(x)=Inf
      return( ifelse(ax==Inf, 0, 
                pmax(.Machine$double.eps, 
                  2-2*F0(ax, param))) )   
    }
  }


  return(list(
         f0 = f0, 
         F0=F0,
         iqr=iqr,
         get.pval = get.pval,
         get.support = get.support
  ))
}

### grenander.R  (2007-06-13)
###
###     Grenander Density Estimator
###
### Copyright 2006-2007 Korbinian Strimmer 
###
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA


grenander = function(F, type=c("decreasing", "increasing"))
{
  if( !any(class(F) == "ecdf") ) stop("ecdf object required as input!")

  type <- match.arg(type)

  if (type == "decreasing")
  {
    # find least concave majorant of ECDF
    ll = gcmlcm(environment(F)$x, environment(F)$y, type="lcm")
  }
  else
  {
    # find greatest convex minorant of ECDF
    l = length(environment(F)$y)
    ll = gcmlcm(environment(F)$x, c(0,environment(F)$y[-l]), type="gcm")
  }

  f.knots = ll$slope.knots
  f.knots = c(f.knots, f.knots[length(f.knots)])

  g = list(F=F,
       x.knots=ll$x.knots,
       F.knots=ll$y.knots,
       f.knots=f.knots)

  class(g) <- "grenander"
  
  return(g)
}

plot.grenander <- function(x, ...)
{
  if (x$f.knots[1] > x$f.knots[2])
    main = "Grenander Decreasing Density"
  else
    main = "Grenander Increasing Density"

  par(mfrow=c(1,2))

  plot(x$x.knots, x$f.knots, type="s", xlab="x", ylab="fn(x)",
     main=main, col=4, lwd=2, ...)
 
  plot(x$F, do.points=FALSE)
  lines(x$x.knots, x$F.knots, type='l', col=4, lwd=2)


  par(mfrow=c(1,1))
}

### gcmlcm.R  (2011-03-19)
###
###
###     Greatest Convex Minorant (GCM) and Least Concave Majorant (LCM) 
###
###
### Copyright 2006-2011 Korbinian Strimmer 
###
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA



# find greatest convex minorant (gcm) or  
# least concave majorant (lcm)

gcmlcm = function(x, y, type=c("gcm", "lcm"))
{
  type=match.arg(type)
  
  if (is.unsorted(x))
    stop("The x values must be arranged in sorted order!")
  
  if (any(duplicated(x)))
    stop("No duplicated x values allowed!")

  ########
  
  dx = diff(x)
  dy = diff(y)

  rawslope = dy/dx

  # make sure there are no Inf in rawslope
  rawslope[rawslope == Inf] <- .Machine$double.xmax
  rawslope[rawslope == -Inf] <- -.Machine$double.xmax

  if (type == "gcm") slope <- pvt.isoMean(rawslope, dx)
  if (type == "lcm") slope <- -pvt.isoMean(-rawslope, dx)

  # remove duplicate slopes
  keep = !duplicated(slope)
  x.knots = x[c(keep, TRUE)] # also keep last point
  dx.knots = diff(x.knots)
  slope.knots = slope[keep]

  y.knots = y[1]+c(0, cumsum(dx.knots*slope.knots))
  
  list(x.knots=x.knots,
       y.knots=y.knots,
       slope.knots=slope.knots)
}

### fdrtool.R  (2013-09-15)
###
###    Estimate (Local) False Discovery Rates For Diverse Test Statistics
###    
###
### Copyright 2007-13 Korbinian Strimmer
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA



 
fdrtool = function(x, 
  statistic=c("normal", "correlation", "pvalue"),
  #statistic=c("normal", "correlation", "pvalue", "studentt"),
  plot=TRUE, color.figure=TRUE, verbose=TRUE, 
  cutoff.method=c("fndr", "pct0", "locfdr"),
  pct0=0.75)
{
  statistic = match.arg(statistic)
  cutoff.method = match.arg(cutoff.method)
  
  if ( is.vector(x) == FALSE )
  	stop("input test statistics must be given as a vector!")
 
  if ( length(x) < 200 ) warning("There may be too few input test statistics for reliable FDR calculations!")
  if (statistic=="pvalue")
  {
    if (max(x) > 1 | min(x) < 0) 
      stop("input p-values must all be in the range 0 to 1!")
  }

  
#### step 1 ####

  if(verbose) cat("Step 1... determine cutoff point\n")

  # determine cutoff point for censoring

  if (cutoff.method=="pct0")
  {
    # use specified quantile

    if(statistic=="pvalue") x0 = quantile(x, probs=1-pct0)
    else x0 = quantile(abs(x), probs=pct0)
  }
  else if ( cutoff.method=="locfdr" & (statistic=="normal" | statistic=="correlation") )
  {
    # use same procedure as in locfdr R package (due to Brit Katzen-Turnbull)

    if(statistic=="normal") z = x
    if(statistic=="correlation") z = atanh(x)

    iqr = as.double(diff(quantile(z, probs=c(.25, .75))))
    sdhat = iqr/(2*qnorm(.75))
    N = length(z)
    # b = 3.55-.44*log(N, 10)                               # locfdr 1.1-3
    b = ifelse(N > 500000, 1,  4.3 * exp(-0.26*log(N,10)) ) # locfdr 1.1-6
    z0 = b*sdhat
    
    if(statistic=="normal") x0 = z0
    if(statistic=="correlation") x0 = tanh(z0)
  }
  else
  {
    if(cutoff.method=="locfdr")
    warning("cutoff.method=\"locfdr\" only available for normal and correlation statistic.")

    # control false nondiscovery rate

    x0 = fndr.cutoff(x, statistic)
  }



#### step 2 ####

  if(verbose) cat("Step 2... estimate parameters of null distribution and eta0\n")

  cf.out <- censored.fit(x=x, cutoff=x0, statistic=statistic)
# cf.out looks as follows for p-values
#     cutoff N0      eta0    eta0.var
#[1,]   0.96 64 0.3730473 0.002141996
# the other test statistics have two more colums containing the scale parameter

  if (statistic=="pvalue")
    scale.param = NULL
  else
    scale.param <- cf.out[1,5] # variance parameter

  eta0 = cf.out[1,3]

#### step 2 ####

  if(verbose) cat("Step 3... compute p-values and estimate empirical PDF/CDF\n")

  nm = get.nullmodel(statistic)
  pval = nm$get.pval(x, scale.param)

  # determine cumulative empirical distribution function (pvalues)
  ee <- ecdf.pval(pval, eta0=eta0)

  g.pval <- grenander(ee)

  #cat("DEBUG: Grenander eta0=", g.pval$f.knots[length(g.pval$f.knots)], "\n")
  #cat("DEBUG: estimated eta0=", eta0 , "\n\n")

  # mixture density and CDF  
  f.pval = approxfun( g.pval$x.knots,  g.pval$f.knots, method="constant", rule=2)
  f0.pval = function(x) return( ifelse(x > 1 | x < 0, 0, rep(1, length(x))) )  

  F.pval = approxfun( g.pval$x.knots,  g.pval$F.knots, method="linear", 
           yleft=0, yright=g.pval$F.knots[length(g.pval$F.knots)])
  F0.pval = function(x) return( ifelse(x > 1, 1, ifelse(x < 0, 0, x )) )

  #fdr.pval = function(p) pmin( eta0   / f.pval(p), 1) # eta0*f0/ f
  fdr.pval = function(p)
  {
    p[ p == .Machine$double.eps ] = 0
    pmin( eta0   / f.pval(p), 1) # eta0*f0/ f
  }

  Fdr.pval = function(p) pmin( eta0*p / F.pval(p), 1) # eta0*F0/ F
  

#### step 4 ####

  if(verbose) cat("Step 4... compute q-values and local fdr\n")

  qval <- Fdr.pval(pval) 
  lfdr <- fdr.pval(pval)


 
#### return results ####


  result = list(pval=pval, qval=qval, lfdr=lfdr, 
             statistic=statistic, param=cf.out)

  if (plot)
  {
    if(verbose) cat("Step 5... prepare for plotting\n")

    ##############
    # zeta > 0 in the following
     

    
 
    if(statistic=="pvalue")
    {
      f0 <- function(zeta) return( nm$f0(zeta, scale.param) ) 
      F0 <- function(zeta) return( nm$F0(zeta, scale.param) )
      get.pval <- function(zeta) return( nm$get.pval(1-zeta, scale.param) )
      x0 = 1-x0
    } 
    else
    {
      f0 <- function(zeta) return( 2*nm$f0(zeta, scale.param)  ) 
      F0 <- function(zeta) return( 2*nm$F0(zeta, scale.param)-1  )
      get.pval <- function(zeta) return( nm$get.pval(zeta, scale.param) )
    }

    fdr = function(zeta)  fdr.pval(get.pval(zeta)) 
    Fdr = function(zeta)  Fdr.pval(get.pval(zeta)) 
     
    F   = function(zeta)  1-eta0*get.pval(zeta)/Fdr(zeta) 
    FA  = function(zeta)  (F(zeta)-eta0*F0(zeta))/(1-eta0)		

    f   = function(zeta)  eta0*(f0(zeta))/fdr(zeta) 
    fA  = function(zeta)  (f(zeta)-eta0*f0(zeta))/(1-eta0)		

   
    ##############

    ax = abs(x) 
    if (statistic=="pvalue") ax = 1-ax  # reverse p-val plot 

    xxx = seq(0, max(ax), length.out=500)
    
    ll = pvt.plotlabels(statistic, scale.param, eta0)

    par(mfrow=c(3,1))

    if (color.figure)
      cols = c(2,4) # colors for f0,F0 and fA,FA
    else
      cols = c(1,1)

    hist(ax, freq=FALSE, bre=50,
      main=ll$main, xlab=ll$xlab, cex.main=1.8)
    lines(xxx, eta0*f0(xxx), col=cols[1], lwd=2, lty=3 )
    lines(xxx, (1-eta0)*fA(xxx), col=cols[2], lwd=2 )
    if (statistic=="pvalue") 
      pos1 = "topleft" else pos1="topright"
    legend(pos1, 
      c("Mixture", "Null Component", "Alternative Component"), 
      lwd=c(1, 2, 2), col=c(1,cols), lty=c(1,3,1), bty="n", cex=1.5)
 
    plot(xxx, F(xxx), lwd=1, type="l", ylim=c(0,1),
      main="Density (first row) and Distribution Function (second row)",
      xlab=ll$xlab, ylab="CDF", cex.main=1.5)
    lines(xxx, eta0*F0(xxx), col=cols[1], lwd=2, lty=3)
    lines(xxx, (1-eta0)*FA(xxx), col=cols[2], lwd=2)
    
    # DEBUG show cutoff in green line
    #lines(c(x0,x0),c(0,1), col=3)

    plot(xxx, Fdr(xxx), type="l", lwd=2, ylim=c(0,1),
      main="(Local) False Discovery Rate", ylab="Fdr and fdr",
      xlab=ll$xlab, lty=3, cex.main=1.5)
    lines(xxx, fdr(xxx), lwd=2)
    if (eta0 > 0.98) 
      pos2 = "bottomleft" else pos2="topright"
    legend(pos2, 
      c("fdr (density-based)", "Fdr (tail area-based)"), 
      lwd=c(2,2), lty=c(1,3), bty="n", cex=1.5)

    # DEBUG show cutoff in green line
    #lines(c(x0,x0),c(0,1), col=3)

    par(mfrow=c(1,1))

    rm(ax)
  }

  if(verbose) cat("\n")

  return(result)
}


#####

## create labels for plots
pvt.plotlabels <- function(statistic, scale.param, eta0)
{
   if (statistic=="pvalue")
   {
     main = paste("Type of Statistic: p-Value (eta0 = ", round(eta0, 4), ")", sep="")
     xlab ="1-pval"
   }

   if (statistic=="studentt")
   {
     df = scale.param 
     main = paste("Type of Statistic: t-Score (df = ", round(df,3),
                       ", eta0 = ", round(eta0, 4), ")", sep="")
     xlab = "abs(t)"
   }

   if (statistic=="normal")
   {
     sd = scale.param 
     main = paste("Type of Statistic: z-Score  (sd = ", round(sd,3),
                       ", eta0 = ", round(eta0, 4), ")", sep="")
     xlab = "abs(z)"
   }

   if (statistic=="correlation")
   {
     kappa =scale.param      
     main = paste("Type of Statistic: Correlation (kappa = ", round(kappa,1),
                       ", eta0 = ", round(eta0, 4), ")", sep="")
     xlab = "abs(r)"
   }

   return(list(main=main, xlab=xlab))
}

### censored.fit.R  (2009-11-19)
###
###     Fit Null Distribution To Censored Data by Maximum Likelihood
###
### Copyright 2006-08 Korbinian Strimmer 
###
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA


# estimate parameters of null distribution 
# using using truncated distributions
# available null distributions
# - normal (with mean zero)
# - correlation (with rho zero)
# - student t
# - uniform


censored.fit = function(x, cutoff,
   statistic=c("normal", "correlation", "pvalue", "studentt"))
{
    statistic = match.arg(statistic)
    cutoff = abs(cutoff)

    if ( !is.vector(x) ) stop("x needs to be a vector!")
    
    
    if (statistic=="pvalue")
    {
      result = matrix(nrow=length(cutoff), ncol=4)
      colnames(result)= c("cutoff", "N.cens", "eta0", "eta0.SE")
    }
    else
    {
      result = matrix(nrow=length(cutoff), ncol=6)
      colnames(result)= c("cutoff", "N.cens", "eta0", "eta0.SE", "sd", "sd.SE")
    }
    if (statistic=="correlation") colnames(result)[5] = "kappa"
    if (statistic=="studentt") colnames(result)[5] = "df"
    if (statistic=="correlation") colnames(result)[6] = "kappa.SE"
    if (statistic=="studentt") colnames(result)[6] = "df.SE"

    
    for (i in 1:length(cutoff))
    {
      x0 = cutoff[i]
      result[i,1] = x0

      out = pvt.fit.nullmodel(x, x0, statistic=statistic)
      result[i,2] = out$N.cens
      result[i,3] = out$eta0
      result[i,4] = out$eta0.SE

      if (statistic!="pvalue")
      {
         result[i,5] = out$param
         result[i,6] = out$param.SE
      }
    }

    return(result)
}

### helper functions


# Richardson extrapolation approximation 
# for numerical computation of curvature
num.curv = function(x, fun) 
{
  macheps = .Machine$double.eps
  h = max( 1e-4, macheps^(1/4)*abs(x) )

  w = c(-1/12,4/3,-5/2,4/3,-1/12) 
  xd = x + h*c(-2,-1,0,1,2)
 
  return( sum(w*fun(xd))/h^2 )
}



### internal functions 

pvt.fit.nullmodel = function(x, x0, statistic)
{
  N = length(x)

  if (statistic=="pvalue")
    x.cens = x[ x >= x0 ]
  else
    x.cens = x[ abs(x) <= x0 ] 

  N.cens = length(x.cens) 
  if (N.cens > N) stop("N must be larger or equal to the size of the censored sample!")
  if (N.cens < 10) 
    warning(paste("Censored sample for null model estimation has only size", 
      length(x.cens), "!"), call.=FALSE)

  #if (N.cens < 2) 
  #  stop(paste("Adjust cutoff point - censored sample more null model has only size", 
  #    length(x.cens), "!"), call.=FALSE)


  ##############

  nm = get.nullmodel(statistic)

  
  # negative log-likelihood function (truncated density)
  nlogL = function(pp) 
  {
    out = rep(0, length(pp))
    for (i in 1:length(pp))
    {
      out[i] = length(x.cens)*log(1-nm$get.pval(x0, pp[i]))-
               sum(nm$f0(x.cens, pp[i], log=TRUE))
    }
    return(out)
  } 

  ##############
  
  # estimate parameters of null model
  if (statistic!="pvalue")
  {
    start = iqr.fit(x.cens, statistic) # start value for scale parameter
    
    #sup = nm$get.support()
    #opt.out = nlminb( start, nlogL, lower=sup[1], upper=sup[2] )
    #sc.param = opt.out$par[1]
   
    sup = nm$get.support()
    lo = max( start/1000, sup[1])
    up = min( start*1000, sup[2])
    sc.param = optimize(nlogL, lower=lo, upper=up)$minimum

    sc.var = 1/num.curv(sc.param,nlogL) # inverse curvature of negative logL
    
    if(is.na(sc.var)) 
    {
       sc.var = 0
       warning("Variance of scale parameter set to zero due to numerical problems")
    }
    if(sc.var < 0) 
    {
       sc.var = 0
       warning("Variance of scale parameter set to zero due to numerical problems")
    }
    sc.SE = sqrt(sc.var)
  }
  else
  {
    sc.param = NULL # no scale parameter
    sc.SE = NULL    
  }

  # ML estimate of eta0
  m = 1-nm$get.pval(x0, sc.param)
  th = N.cens/N
  eta0 = min(1, th / m )
  #eta0 = th / m 
  eta0.SE = sqrt( th*(1-th)/(N*m*m) )

  rm(x.cens)

  return(
    list(N.cens=N.cens,
         eta0=eta0,              # proportion
         eta0.SE=eta0.SE,        # corresponding standard error
         param=sc.param,         # scale parameter
         param.SE=sc.SE          # corresponding standard error
        )
  )
}

### dcor0.R  (2007-01-09)
###
###    Distribution of the Correlation Coefficient (rho=0) 
###    and Related Functions
###    
###
### Copyright 2003-07 Korbinian Strimmer
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA



# density function
dcor0 <- function (x, kappa, log = FALSE)
{
  t <- r2t(x, kappa)
  df <- kappa-1
  vol <- sqrt(df)*(1-x^2)^(-3/2)
  if (log) 
    d <-  dt(t, df=df, log=log) + log(vol)
  else
    d <-  dt(t, df=df, log=log) * vol
 
  return(d) 
}

# distribution function
pcor0 <- function(q, kappa, lower.tail=TRUE, log.p=FALSE)
{
  t <- r2t(q, kappa)
  df <- kappa-1
  p <- pt(t, df=df, lower.tail = lower.tail, log.p = log.p)

  return(p)
}

# quantile function
qcor0 <- function(p, kappa, lower.tail=TRUE, log.p=FALSE)
{
  df <- kappa-1
  r <- t2r(qt(p, df=df, lower.tail = lower.tail, log.p = log.p), df)
  
  return(r)
}

# random number generator
rcor0 <- function(n, kappa)
{
  df <- kappa-1
  r <- t2r(rt(n, df),df)
  
  return(r)
}


### conversion from r to t statistic (and vice versa)

r2t <- function(r, kappa)
{
  t = r*sqrt((kappa-1)/(1-r*r)) 

  return(t) # df = kappa-1
} 

t2r <- function(t, df)
{
  r = t/sqrt(t*t+df)  

  return(r) # kappa = df+1
} 

### halfnorm.R  (2007-01-01)
###
###     Half-Normal Distribution
###
### Copyright 2006-2007 Korbinian Strimmer 
###
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA



dhalfnorm <- function(x, theta=sqrt(pi/2), log=FALSE)
{
  sd.norm = theta2sd(theta)
  
  if (log)
    d = ifelse(x<0, -Inf, log(2)+dnorm(x, mean=0, sd=sd.norm, log=TRUE))
  else
    d = ifelse(x<0, 0,  2*dnorm(x, mean=0, sd=sd.norm)) 

  return(d)
}

phalfnorm <- function(q, theta=sqrt(pi/2), lower.tail=TRUE, log.p=FALSE)
{
  sd.norm = theta2sd(theta)
  
  p = ifelse(q < 0, 0, 2*pnorm(q, mean=0, sd=sd.norm)-1)
  if (lower.tail == FALSE) p = 1-p
  if(log.p) p = log(p)
  
  return( p )
}

qhalfnorm <- function(p, theta=sqrt(pi/2), lower.tail=TRUE, log.p=FALSE)
{
  sd.norm = theta2sd(theta)
  
  if (log.p) p = exp(p)
  if (lower.tail == FALSE) p = 1-p
  q = ifelse(p < 0, NaN, qnorm((p+1)/2, mean=0, sd=sd.norm))
  
  return(q)
}

rhalfnorm <- function(n, theta=sqrt(pi/2))
{
  sd.norm = theta2sd(theta)
  return( abs(rnorm(n, mean=0, sd=sd.norm)) )
}


# conversion between standard deviation of normal distribution
# and theta parameter of corresponding half-normal distribution

sd2theta <- function(sd)
{
  return(sqrt(pi/2)/sd)
}

theta2sd <- function(theta)
{
  return(sqrt(pi/2)/theta)
}

### monoreg.R  (2020-12-23)
###
###     Monotone Regression
###
### Copyright 2006-2020 Korbinian Strimmer 
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA


# monotone regression

monoreg = function(x, y=NULL, w=rep(1, length(x)), 
    type=c("isotonic", "antitonic"))
{ 
    # get x-y coordinates
    xy = xy.coords(x,y)
    x = xy$x
    y = xy$y
    
    if ( any(w == 0) )
    {
        warning("Weights of 0 detected!\nCorresponding observations are being removed.")
        x = x[w != 0]
        y = y[w != 0]
        w = w[w != 0]
    } 
   
    # remove duplicated x values
    xvals = unique(x)
    lx = length(xvals)
    
    if (lx == length(x)) # x values are all unique
    {
       wts   = w
       yvals = y 
    }
    else # merge duplicated x values
    {   
       wts = rep(NA, lx)
       yvals = rep(NA, lx)
       for (i in 1:lx)
       {
         idx = which(xvals[i]==x)
	 
	 if (length(idx) > 1)
	 {
	    warning("Duplicated x value (x=", xvals[i], ") detected!\nThe corresponding weights and y values will be merged.")
	    wi = w[idx]
            wts[i] = sum(wi) # add all weights
            yvals[i] = sum(y[idx]*wi)/wts[i]  # weighted mean
	 }
	 else
	 {
	    wts[i] = w[idx]
	    yvals[i] = xy$y[idx]
	 }
	 
       }
    } 
    
    # sort
    
    ord = order(xvals)
    xvals = xvals[ord]
    yvals = yvals[ord]
    wts = wts[ord]
       
    # perform monotonic regression      
          
    type = match.arg(type)
        
    if (type=="isotonic")
    {
        yf = pvt.isoMean(yvals, wts)
    }
    
    if (type=="antitonic")
    {
	yf = -pvt.isoMean(-yvals, wts)
    }
	
    # output results
    
    out = list(x=xvals, 
               y=yvals, 
	       w=wts,
               yf=yf,
	       type=type,
	       call = match.call())
    class(out) = c("monoreg")
    
    return(out)
}

fitted.monoreg = function(object, ...) object$yf
residuals.monoreg = function(object, ...) object$y - fitted(object)


plot.monoreg = function(x, main, main2,
     plot.type = c("single", "row.wise", "col.wise"), ...)
{
  plot.type = match.arg(plot.type)
  if (plot.type=="row.wise") par(mfrow=c(2,1))
  if (plot.type=="col.wise") par(mfrow=c(1,2))
  
  
  if (missing(main))
  {
     if (x$type=="isotonic")
        main = paste("Isotonic Regression:", deparse(x$call))
     else
        main = paste("Antitonic Regression:", deparse(x$call))
  }

  plot(x$x, x$y, main=main, xlab="x", ylab="y", ...)
  lines(x$x, x$yf, col=4, lty=3)
  points(x$x, x$yf, col=4, pch=20)
  
  if (plot.type != "single")
  { 
  
    if (missing(main2))
    {
      if(x$type == "isotonic")
        main2="Cumulative Sum Diagram and Greatest Convex Minorant"
      else
        main2="Cumulative Sum Diagram and Least Concave Majorant"  
    }
  
    G = c(0, cumsum(x$y*x$w))
    M = c(0, cumsum(x$yf*x$w))
    W = c(0, cumsum(x$w))
  
    plot(W, G, main=main2, 
       xlab="cumsum(w)", ylab="cumsum(y*w)", type="l")
    points(W, G)
    lines(W, M, col=4, lty=3)
    points(W, M, col=4, pch=20)
  
    par(mfrow=c(1,1))
  }
}


######## internal function ###################

pvt.isoMean = function(y, w)
{
  # Input:	y: measured values in a regression setting
  #		w: weights
  # Output: 	vector containing estimated (isotonic) values

  n = length(y)

  if(n == 1)
  {
    return(y)
  }
  else
  {
    ghat = .C("C_isomean",
            as.double(y),
            as.double(w),
            as.integer(n),
            ghat=double(n), PACKAGE="fdrtool")$ghat

    return(ghat)
  }
}

### pval.estimate.eta0.R  (2007-10-11)
###
###     Estimating the Proportion of Null p-Values
###
### Copyright 2003-2007 Korbinian Strimmer 
###
### Parts of this code is adapted from 
### S-PLUS code (c) by Y. Benjamini (available from
### http://www.math.tau.ac.il/~roee/FDR_Splus.txt ) 
### and from R code (c) J.D. Storey (available from 
### http://faculty.washington.edu/~jstorey/qvalue/ )
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA


#Input
#=============================================================================
#p: a vector of p-values 
#method:  method for computing eta0
#lambda:  optional tuning parameter (vector, needed for "bootstrap" and "smoothing") 
#
#  conservative: Benjamini and Hochberg (1995) JRSSB
#  adaptive:     Benjamini and Hochberg (2000) J. Behav. Educ. Statist.
#  bootstrap:    Storey (2002) JRSSB
#  smoother:     Storey and Tibshirani (2003) PNAS
#  quantile:     similar to "smoother", with eta0 assumed to be quantile q (10%)



#Output
#=============================================================================
#eta0: an estimate of the proportion of null p-values




pval.estimate.eta0 <- function(p,
    method=c("smoother", "bootstrap", "conservative", "adaptive", "quantile"),
    lambda=seq(0,0.9,0.05), diagnostic.plot=TRUE, q=0.1)
{
    method <- match.arg(method)
            
    if (method == "conservative") # Benjamini and Hochberg (1995)
    {
        eta0 = 1.0
    }
   
    if (method == "adaptive") # Benjamini and Hochberg (2000)
    {
        m <- length(p)
	sortp <- sort(p)
	s <- sort(1 - sortp)/(1:m)
	
	m0raw <- m
        i <- m
        while(i > 1 && s[i] <= s[i - 1]) i <- i - 1
        if(i > 1)
            m0raw <- 1/s[i - 1]
        else m0raw <- 1/s[1]
        
	m0 <- min(floor(1 + m0raw), m)
        
	eta0 <- m0/m      
    }

    if(method == "bootstrap" || method == "smoother" || method == "quantile")
    {
      # for the remaining methods we a set of p-value thresholds
      if (length(lambda) < 4)
        stop("At least 4 values in lambda tuning vector required")
    
      e0.vec <- rep(0,length(lambda))
      for(i in 1:length(lambda))
      {
        e0.vec[i] <- mean(p >= lambda[i])/(1-lambda[i])
      }
    }

    if(method == "quantile")
    {
      e0.vec <- pmax(0, pmin(1, e0.vec))
      eta0 <- as.double(quantile( e0.vec, probs=c(q))) 
    }
           
    if(method == "bootstrap") # Storey (2002) JRSSB
    {
      m <- length(p)
      mineta0 <- min(e0.vec)
      mse <- rep(0,length(lambda))
      eta0.boot <- rep(0,length(lambda))
      for(i in 1:100) 
      {
        p.boot <- sample(p,size=m,replace=TRUE)
        for(i in 1:length(lambda)) 
        {
          eta0.boot[i] <- mean(p.boot>lambda[i])/(1-lambda[i])
        }
        mse <- mse + (eta0.boot-mineta0)^2
      }
      idx <- which.min(mse)[1]
      lambda.min <- lambda[idx]
      eta0 <- max(0, min(e0.vec[idx], 1))

      if (diagnostic.plot)
      {
        dev.new() # open new plot window
        par(mfrow=c(2,1))
        plot(lambda, e0.vec, ylab="eta0")
        points( lambda.min, eta0, pch=20, col=2 )
        plot(lambda, mse)
        points( lambda.min, min(mse), pch=20, col=2 )
        par(mfrow=c(1,1))
      }
    }    
  
    if(method == "smoother") # Storey and Tibshirani (2003) PNAS
    {
      e0.spline <- smooth.spline(lambda, e0.vec, df=3)
      eta0 <- max(0,min( predict(e0.spline, x=max(lambda))$y, 1))
      
      if (diagnostic.plot)
      {
        dev.new() # open new plot window
        plot(lambda, e0.vec, main="Smoothing Curve Employed For Estimating eta0", 
          xlab="lambda", ylab="eta0")
        lines( e0.spline )
        points( max(lambda), eta0, pch=20, col=2 )
      }

    } 
       
    if (diagnostic.plot)
    {
      dev.new() # open new plot window
  
      info0 = paste("method =", method)
      info1 = paste("eta0 =", round(eta0, 4))
  
      h = hist(p, freq=FALSE, bre="FD", 
                    main="Diagnostic Plot:  Distribution of p-Values and Estimated eta0", xlim=c(0,1), 
                    xlab="p-values")
      y0 = dunif(h$breaks)*eta0
      lines(h$breaks, y0, col=2)
      maxy = max(h$density)
  
      text(0.5, 5/6*maxy, info0) 
      text(0.5, 4/6*maxy, info1, col=2)  
    }
     


    return(eta0) 
}

### hc.score.R  (2012-07-25)
###
###     Compute empirical higher criticism score and threshold from p-values
###
### Copyright 2010-2012 Bernd Klaus and Korbinian Strimmer 
###
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA



# compute empirical HC scores from p-values
hc.score = function(pval)
{
  if (max(pval) > 1 | min(pval) < 0) 
      stop("input p-values must all be in the range 0 to 1!")

  d = length(pval)
  F = rank(pval, ties.method="max")/d
  
  v = F*(1-F)/d # variance
  v[v==0] = min( v[v > 0] ) # just to make sure we have no zero variance
  HC = abs(F-pval)/sqrt(v)

  return( HC )
}


# determine HC decision threshold from p-values

# pval:    p-values
# alpha0:  look only at a fraction alpha0 of the p-values (default: 1)
# plot:    produce a plot of HC curve

hc.thresh = function(pval, alpha0=1, plot=TRUE)
{
  spval = sort(pval)    # sort p-values for plotting
  hcval = hc.score(spval)

  alpha0.idx = 1:ceiling(alpha0*length(hcval)) # the fraction of HC values to maximize over

  hcstat.idx = which.max( hcval[alpha0.idx] ) # idx of maximum HC 
  hcstat      = hcval[hcstat.idx]             # maximum HC
  hcstat.pval = spval[hcstat.idx]             # pval of maximum HC 

  if (plot)
  {
     plot(spval[alpha0.idx], hcval[alpha0.idx], type="l", xlab="ordered p-values", ylab="HC score")
    lines( c(hcstat.pval, hcstat.pval), c(0, hcstat)  , col=2)
  }

  return(hcstat.pval)
}

### ecdf.pval.R  (2007-06-15)
###
###    Estimate Empirical Density of p-Values 
###    
###
### Copyright 2007 Korbinian Strimmer
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA


# empirical cumulative distribution of p-values,
# constrained such that the known fraction eta0 of null p-values 
# is taken into account
ecdf.pval <- function (x, eta0=1) 
{
    # compute empirical CDF as usual
    x = sort(x)
    n = length(x)
    if (n < 1) 
        stop("'x' must have 1 or more non-missing values")
    vals = sort(unique(x))
    F.raw = cumsum(tabulate(match(x, vals)))/n
    
    # control upper bound of F:
    # make sure that the maximum slope of (Grenander) F is eta0
    F.raw = pmin(F.raw, 1-eta0*(1-vals) ) 

    # control lower bound of F: 
    # make sure that (Grenander F) >= eta0*vals
    F.raw = pmax(F.raw, eta0*vals) 

    # if necessary add an atom at 1 to make it a proper CDF
    if (vals[length(vals)] != 1)
    {
       F.raw = c(F.raw, 1)
       vals = c(vals, 1)
    }

    # if necessary also add an atom at 0 with weight zero to get support [0,1]
    if (vals[1] != 0)
    {
       F.raw = c(0, F.raw)
       vals = c(0, vals)
    }

    # finally, modify F such that the last slope of the Grenander F 
    # is *exactly* eta0
    i = length(vals)-1
    F.raw[i] = 1-eta0*(1-vals[i])
    
    rval <- approxfun(vals, F.raw, 
        method = "constant", yleft = 0, yright = 1, f = 0, ties = "ordered")
    class(rval) = c("ecdf", "stepfun", class(rval))
    attr(rval, "call") <- sys.call()
    rval
}

### approximate.fit.R  (2007-10-19)
###
###     First Guess of Null Model Parameters
###
### Copyright 2007 Korbinian Strimmer 
###
###
### This file is part of the `fdrtool' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 3, or at your option, any later version,
### incorporated herein by reference.
### 
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
### 
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA


# first guess of null model parameters
approximate.fit = function(x, statistic=c("normal", "correlation", "pvalue", "studentt"), lambda=seq(0,0.9,0.05))
{
  statistic <- match.arg(statistic)
  nm = get.nullmodel(statistic)

  param = iqr.fit(x, statistic)  # approximate estimate of scale parameter
  pval = nm$get.pval(x, param)   # compute corresponding p-values

  # estimate eta0
  eta0 = pval.estimate.eta0(pval, method="quantile", q=0.1, diagnostic.plot=FALSE, lambda=lambda) 

  return(list(eta0=eta0, param=param))
}


# find robust estimate of scale by fitting IQR
iqr.fit = function(x, statistic=c("normal", "correlation", "pvalue", "studentt"))
{
  statistic <- match.arg(statistic)
  nm = get.nullmodel(statistic)
  
  # observed interquantile range
  iqr.obs = as.double(diff(quantile(x, probs=c(.25, .75))))
  
  if (statistic=="pvalue")
  {
    param = NULL
  }
  else if (statistic=="normal")
  {
    param = iqr.obs/(2*qnorm(.75))
  }
  else
  {
    mfun = function(param) return( (nm$iqr(param)-iqr.obs)^2 )
    supp = nm$get.support()
    param = optimize(mfun, supp)$minimum  
  }

  return(param)
}

\name{grenader}
\alias{grenander}
\alias{plot.grenander}

\title{Grenander Estimator of a Decreasing or Increasing Density}
\description{
  The function \code{grenander} computes the Grenander estimator
  of a one-dimensional decreasing or increasing density. 
}
\usage{
grenander(F, type=c("decreasing", "increasing"))
}
\arguments{
  \item{F}{an \code{\link{ecdf}} containing the empirical cumulative density.}
  \item{type}{specifies whether the distribution is decreasing (the default) or increasing.}
}
\details{
  The Grenander (1956) density estimator is given by the slopes of the 
  least concave majorant (LCM) of the empirical distribution function (ECDF).
  It is a decreasing piecewise-constant function and can be shown to be the 
  non-parametric maximum likelihood estimate (NPMLE) under the assumption
  of a decreasing density (note that the ECDF is the NPMLE without
  this assumption).  Similarly, an increasing density function is obtained
  by using the greatest convex minorant (GCM) of the ECDF.
}
\value{
  A list of class \code{grenander} with the following components:
  
  \item{F}{the empirical distribution function specified as input.   } 
  \item{x.knots}{x locations of the knots of the least concave majorant of the ECDF.}
  \item{F.knots}{the corresponding y locations of the least concave majorant of the ECDF.}
  \item{f.knots}{the corresponding slopes (=density).}

}
\author{
  Korbinian Strimmer (\url{http://www.strimmerlab.org}).
}
\references{

  Grenander, U. (1956). On the theory of mortality measurement, Part II.  
  \emph{Skan. Aktuarietidskr},
  \bold{39}, 125--153.
}
\seealso{\code{\link{ecdf}}, \code{\link{gcmlcm}}, \code{\link{density}}.}

\examples{
# load "fdrtool" library
library("fdrtool")

# samples from random exponential variable 
z = rexp(30,1)
e = ecdf(z)
g = grenander(e)
g

# plot ecdf, concave cdf, and Grenander density estimator (on log scale)
plot(g, log="y") 

# for comparison the kernel density estimate
plot(density(z)) 

# area under the Grenander density estimator 
sum( g$f.knots[-length(g$f.knots)]*diff(g$x.knots) )
}
\keyword{univar}

\name{dcor0}
\alias{dcor0}
\alias{pcor0}
\alias{rcor0}
\alias{qcor0}

\title{Distribution of the Vanishing Correlation Coefficient (rho=0) and Related Functions}

\usage{
dcor0(x, kappa, log=FALSE)
pcor0(q, kappa, lower.tail=TRUE, log.p=FALSE)
qcor0(p, kappa, lower.tail=TRUE, log.p=FALSE)
rcor0(n, kappa)
}

\arguments{
	\item{x,q}{vector of sample correlations}
        \item{p}{vector of probabilities}
	\item{kappa}{the degree of freedom of the distribution (= inverse variance)}
	\item{n}{number of values to generate. If n is a vector, length(n)
                values will be generated}
	\item{log, log.p}{logical vector; if TRUE, probabilities p are given as log(p)}
	\item{lower.tail}{logical vector; if TRUE (default), probabilities are \eqn{P[R <= r]},
	                  otherwise, \eqn{P[R > r]}}
}

\description{
  The above functions describe the distribution of the Pearson correlation 
  coefficient \code{r} assuming that there is no correlation present (\code{rho = 0}).
 
  Note that the distribution has only a single parameter: the degree 
  of freedom \code{kappa}, which is equal to the inverse of the variance of the distribution.
  
  The theoretical value of  \code{kappa} depends both on the sample size \code{n} and the number 
  \code{p} of considered variables.  If a simple correlation coefficient between two
  variables  (\code{p=2})  is considered the degree of freedom equals \code{kappa = n-1}.
  However, if a partial correlation coefficient is considered (conditioned on \code{p-2} remaining 
  variables) the degree of freedom is \code{kappa = n-1-(p-2) = n-p+1}.
}

\details{
  For density and distribution functions as well as a corresponding random number generator
  of the correlation coefficient for arbitrary non-vanishing correlation \code{rho} please refer to the
  \code{SuppDists} package by  Bob Wheeler \email{bwheeler@echip.com} (available on CRAN).
  Note that the parameter \code{N} in his \code{dPearson} function corresponds to  \code{N=kappa+1}.
}

\value{
   \code{dcor0} gives the density,  \code{pcor0} 
  gives the distribution function, \code{qcor0} gives
  the quantile function, and \code{rcor0} generates random deviates.
}

\seealso{\code{\link{cor}}.}

\author{
   Korbinian Strimmer (\url{http://www.strimmerlab.org}).
}

\examples{
# load fdrtool library
library("fdrtool")

# distribution of r for various degrees of freedom
x = seq(-1,1,0.01)
y1 = dcor0(x, kappa=7)
y2 = dcor0(x, kappa=15)
plot(x,y2,type="l", xlab="r", ylab="pdf",
  xlim=c(-1,1), ylim=c(0,2))
lines(x,y1)

# simulated data
r = rcor0(1000, kappa=7)
hist(r, freq=FALSE, 
  xlim=c(-1,1), ylim=c(0,5))
lines(x,y1,type="l")

# distribution function
pcor0(-0.2, kappa=15)
}
\keyword{distribution}

\name{monoreg}
\alias{monoreg}
\alias{plot.monoreg}
\alias{fitted.monoreg}
\alias{residuals.monoreg}

\title{Monotone Regression: Isotonic Regression and Antitonic Regression}

\usage{
monoreg(x, y=NULL, w=rep(1, length(x)), type=c("isotonic", "antitonic"))
}

\description{
  \code{monoreg} performs monotone regression (either isotonic
  or antitonic) with weights. 
}

\arguments{
  \item{x, y}{coordinate vectors of the regression
    points.  Alternatively a single \dQuote{plotting} structure can be
    specified: see \code{\link{xy.coords}}.}
  \item{w}{data weights (default values: 1).}
    
 
  \item{type}{fit a monotonely increasing ("isotonic") or 
  monotonely decreasing ("antitonic") function.}
}

\details{

 \code{monoreg} is similar to \code{\link{isoreg}}, with the addition
 that \code{monoreg} accepts weights.

 If several identical \code{x} values are given as input, the 
 corresponding \code{y} values and the
 weights \code{w} are automatically merged, and a warning is issued.
 
 The \code{plot.monoreg} function optionally plots the cumulative
 sum diagram with the greatest convex minorant (isotonic regression)
 or the least concave majorant (antitonic regression), see the
 examples below. 
}

\value{
  A list with the following entries:
  
  \item{x}{the sorted and unique x values}
  \item{y}{the corresponding y values}
  \item{w}{the corresponding weights}
  \item{yf}{the fitted y values}
  \item{type}{the type of monotone regression ("isotonic" or "antitonic"}
  \item{call}{the function call}
}

\references{
 
 Robertson, T., F. T. Wright, and R. L. Dykstra. 1988.  Order restricted
 statistical inference. John Wiley and Sons.

}
\author{
  Korbinian Strimmer (\url{http://www.strimmerlab.org}).
  
  Part of this function is C code that has been ported from
  R code originally written by Kaspar Rufibach. 
}
\seealso{\code{\link{isoreg}}.}

\examples{
# load "fdrtool" library
library("fdrtool")


# an example with weights

# Example 1.1.1. (dental study) from Robertson, Wright and Dykstra (1988)
age = c(14, 14, 8, 8, 8, 10, 10, 10, 12, 12, 12)
size = c(23.5, 25, 21, 23.5, 23, 24, 21, 25, 21.5, 22, 19)

mr = monoreg(age, size)

# sorted x values
mr$x # 8 10 12 14
# weights and merged y values
mr$w  # 3 3 3 2
mr$y #  22.50000 23.33333 20.83333 24.25000
# fitted y values
mr$yf # 22.22222 22.22222 22.22222 24.25000
fitted(mr)
residuals(mr)

plot(mr, ylim=c(18, 26))  # this shows the averaged data points
points(age, size, pch=2)  # add original data points


###

y = c(1,0,1,0,0,1,0,1,1,0,1,0)
x = 1:length(y)
mr = monoreg(y)

# plot with greatest convex minorant
plot(mr, plot.type="row.wise")  

# this is the same
mr = monoreg(x,y)
plot(mr)

# antitonic regression and least concave majorant
mr = monoreg(-y, type="a")
plot(mr, plot.type="row.wise")  

# the fit yf is independent of the location of x and y
plot(monoreg(x + runif(1, -1000, 1000), 
             y +runif(1, -1000, 1000)) )

###

y = c(0,0,2/4,1/5,2/4,1/2,4/5,5/8,7/11,10/11)
x = c(5,9,13,18,22,24,29,109,120,131)

mr = monoreg(x,y)
plot(mr, plot.type="row.wise")

# the fit (yf) only depends on the ordering of x
monoreg(1:length(y), y)$yf 
monoreg(x, y)$yf 


}
\keyword{regression}
\keyword{smooth}

\name{pval.estimate.eta0}
\alias{pval.estimate.eta0}
\title{Estimate the Proportion of Null p-Values}
\usage{
pval.estimate.eta0(p, method=c("smoother", "bootstrap", "conservative",
   "adaptive", "quantile"), lambda=seq(0,0.9,0.05), 
   diagnostic.plot=TRUE, q=0.1)
}
\description{
  \code{pval.estimate.eta0} estimates the proportion eta0 of null p-values in a given
  vector of p-values. 
}

\arguments{
  \item{p}{vector of p-values}
  \item{method}{algorithm used to estimate the proportion of null p-values.
     Available options are
    "conservative" , "adaptive", "bootstrap", quantile, and "smoother" (default).}
   \item{lambda}{optional tuning parameter vector needed for "bootstrap"
     and "smoothing" methods (defaults to \code{seq(0,0.9,0.05)})
     - see Storey (2002) and Storey and Tibshirani (2003).}
   \item{diagnostic.plot}{if \code{TRUE} (the default) the histogram of the p-values together
          with the estimate of \code{eta0} null line is plotted.  This is useful to
	  visually check the fit of the estimated proportion of null p-values.}  
   \item{q}{quantile used for estimating eta0 - only if \code{method}="quantile"}
  }

\details{

This quantity \code{eta0}, i.e.  the proportion eta0 of null p-values in a given
  vector of p-values, is an important parameter 
  when controlling  the false discovery rate (FDR).  A conservative choice is
  eta0 = 1 but a choice closer to the true value will increase efficiency
  and power 
  - see   Benjamini and Hochberg (1995, 2000) and Storey (2002) for details.

The function \code{pval.estimate.eta0} provides five algorithms: the "conservative"
method always returns eta0 = 1  (Benjamini and Hochberg, 1995), "adaptive"
uses the approach suggested in Benjamini and Hochberg (2000), "bootstrap"
employs the method from Storey (2002), "smoother" uses the smoothing spline
approach in Storey and Tibshirani (2003), and "quantile" is a simplified version
of "smoother".
}
\value{
  The estimated proportion eta0 of null p-values. 
}

\references{
 
 \emph{"conservative" procedure:} Benjamini, Y., and Y. Hochberg (1995)  Controlling the false
 discovery rate: a practical and powerful approach to multiple testing.
 \emph{J. Roy. Statist. Soc. B}, \bold{57}, 289--300.

 \emph{"adaptive" procedure:} Benjamini, Y., and Y. Hochberg (2000) The adaptive control
 of the false discovery rate in multiple hypotheses testing with independent statistics.
 \emph{J. Behav. Educ. Statist.}, \bold{25}, 60--83.
 
 \emph{"bootstrap" procedure:} Storey, J. D. (2002) A direct approach to false
 discovery rates.
 \emph{J. Roy. Statist. Soc. B.}, \bold{64}, 479--498.
 
 \emph{"smoother" procedure:} Storey, J. D., and R. Tibshirani (2003)
  Statistical significance for genome-wide experiments. 
 \emph{Proc. Nat. Acad. Sci. USA}, \bold{100}, 9440-9445.

 \emph{"quantile" procedure:} similar to smoother, except that the lower q
 quantile of all eta0 computed in dependence of lambda is taken as overall estimate of eta0.

}
\author{
  Korbinian Strimmer (\url{http://www.strimmerlab.org}).
  
  Adapted in part from code by Y. Benjamini and J.D. Storey. 
}
\seealso{\code{\link{fdrtool}}.}

\examples{
# load "fdrtool" library and p-values
library("fdrtool")
data(pvalues)


# Proportion of null p-values for different methods
pval.estimate.eta0(pvalues, method="conservative")
pval.estimate.eta0(pvalues, method="adaptive")
pval.estimate.eta0(pvalues, method="bootstrap")
pval.estimate.eta0(pvalues, method="smoother")
pval.estimate.eta0(pvalues, method="quantile")
}
\keyword{htest}

\name{fdrtool-internal}

\alias{pvt.isoMean}
\alias{r2t}
\alias{t2r}

\alias{pvt.nullfunction}
\alias{pvt.plotlabels}
\alias{ecdf.pval}

\alias{num.curv}

\alias{approximate.fit}
\alias{iqr.fit}

\alias{get.nullmodel}
\alias{pvt.fit.nullmodel}




\title{Internal fdrtool Functions}
\description{
  Internal fdrtool functions.
}
\note{
  These are not to be called by the user (or in some cases are just
  waiting for proper documentation to be written).
}
\keyword{internal}

\name{pvalues}
\alias{pvalues}

\title{Example p-Values}

\description{
  This data set contains 4,289 p-values.  These data are used to
  illustrate the functionality of the functions \code{\link{fdrtool}}
  and \code{\link{pval.estimate.eta0}}.
}

\usage{
data(pvalues)
}

\format{
  \code{pvalues} is a vector with 4,289 p-values. 
}


\examples{
# load fdrtool library
library("fdrtool")

# load data set
data(pvalues)

# estimate density and distribution function, 
# and compute corresponding (local) false discovery rates
fdrtool(pvalues, statistic="pvalue")
}
\keyword{datasets}

\name{censored.fit}
\alias{censored.fit}
\alias{fndr.cutoff}

\title{Fit Null Distribution To Censored Data by Maximum Likelihood}

\usage{
censored.fit(x, cutoff, statistic=c("normal", "correlation", "pvalue", "studentt"))
fndr.cutoff(x, statistic=c("normal", "correlation", "pvalue", "studentt"))
}

\description{
  \code{censored.fit} fits  a null distribution 
  to censored data.

  \code{fndr.cutoff} finds a suitable cutoff point based on the 
   (approximate) false non-discovery rate (FNDR).
}

\arguments{
  \item{x}{vector of test statistics.}
  \item{cutoff}{truncation point (this may a single value or a vector).}
  \item{statistic}{type of statistic - normal, correlation, or student t.}
}
\details{

   As null model truncated normal, truncated student t or a truncated
   correlation density is assumed.  The truncation point is specified
   by the cutoff parameter.  All data points whose absolute value
   are large than the cutoff point are ignored when fitting the truncated
   null model via maximum likelihood.  The total number of data points is
   only used to estimate the fraction of null values eta0.
}
\value{
  \code{censored.fit} returns a matrix whose rows contain the estimated parameters and corresponding errors
  for each cutoff point. 

  \code{fndr.cutoff} returns a tentative cutoff point.
}


\seealso{\code{\link{fdrtool}}.}

\examples{
# load "fdrtool" library
library("fdrtool")

# simulate normal data
sd.true = 2.232
n = 5000
z = rnorm(n, sd=sd.true)
censored.fit(z, c(2,3,5), statistic="normal")


# simulate contaminated mixture of correlation distribution
r = rcor0(700, kappa=10)
u1 = runif(200, min=-1, max=-0.7)
u2 = runif(200, min=0.7, max=1)
rc = c(r, u1, u2)

censored.fit(r, 0.7, statistic="correlation")
censored.fit(rc, 0.7, statistic="correlation")

# pvalue example
data(pvalues)
co = fndr.cutoff(pvalues, statistic="pvalue")
co
censored.fit(pvalues, cutoff=co, statistic="pvalue")
}
\keyword{htest}

\name{fdrtool}
\alias{fdrtool}
\title{Estimate (Local) False Discovery Rates For Diverse Test Statistics}
\usage{
fdrtool(x, statistic=c("normal", "correlation", "pvalue"),
  plot=TRUE, color.figure=TRUE, verbose=TRUE, 
  cutoff.method=c("fndr", "pct0", "locfdr"),
  pct0=0.75)
}
\description{
  \code{fdrtool} takes a vector of z-scores (or of correlations, p-values,
  or t-statistics), and estimates for each case both the tail area-based Fdr
  as well as the density-based fdr (=q-value resp. local false discovery rate).
  The parameters of the null distribution are 
  estimated adaptively from the data (except for the case of p-values where
  this is not necessary).
}

\arguments{
  \item{x}{vector of the observed test statistics.}
  \item{statistic}{one of "normal" (default), "correlation", "pvalue".  
         This species the null model.}
  \item{plot}{plot a figure with estimated densities, distribution functions, 
             and (local) false discovery rates.}
  \item{verbose}{print out status messages.}
  \item{cutoff.method}{one of "fndr" (default), "pct0", "locfdr".}
  \item{pct0}{fraction of data used for fitting null model - only if \code{cutoff.method}="pct0"}
  \item{color.figure}{determines whether a color figure or a black and white
        figure is produced (defaults to "TRUE", i.e. to color figure).}

}
\details{
  The algorithm implemented in this function proceeds as follows:

 \enumerate{
   \item  A suitable cutoff point is determined.  If \code{cutoff.method}
          is "fndr" then first an approximate null model is fitted and
          subsequently a cutoff point is sought with false nondiscovery
          rate as small as possible (see \code{\link{fndr.cutoff}}). 
          If \code{cutoff.method} is "pct0"
          then a specified quantile (default value: 0.75) of the data
          is used as the cutoff point.  If \code{cutoff.method} equals
          "locfdr" then the heuristic of the "locfdr" package (version 1.1-6)
          is employed to find the cutoff (z-scores and correlations only).
   \item  The parameters of the null model are estimated from the 
          data using \code{\link{censored.fit}}. This results
          in estimates for scale parameters und and proportion
          of null values (\code{eta0}). 
   \item  Subsequently the corresponding p-values are computed, and
          a modified \code{\link{grenander}} algorithm is employed
          to obtain the overall density and distribution function 
          (note that this respects the estimated \code{eta0}).
   \item  Finally, q-values and local fdr values are computed for each case.
  }

  The assumed null models all have (except for p-values) one free
  scale parameter.  Note that the z-scores and the correlations
  are assumed to have zero mean. 

}
\value{
  A list with the following components:

  \item{pval}{a vector with p-values for each case.} 
  \item{qval}{a vector with q-values (Fdr) for each case.}
  \item{lfdr}{a vector with local fdr values for each case.}
  \item{statistic}{the specified type of null model.} 
  \item{param}{a vector containing the estimated parameters (the null 
               proportion \code{eta0}  and the free parameter of the null model).}  
}

\author{
  Korbinian Strimmer (\url{http://www.strimmerlab.org}).
}
\references{

  Strimmer, K. (2008a).   A unified approach to false discovery 
  rate estimation. BMC Bioinformatics 9: 303.
  <DOI:10.1186/1471-2105-9-303>

  Strimmer, K. (2008b). fdrtool: a versatile R package for estimating 
  local and tail area- based false discovery rates.
  Bioinformatics 24: 1461-1462.
  <DOI:10.1093/bioinformatics/btn209>
}

\seealso{\code{\link{pval.estimate.eta0}}, \code{\link{censored.fit}}.}

\examples{
# load "fdrtool" library and p-values
library("fdrtool")
data(pvalues)


# estimate fdr and Fdr from p-values

data(pvalues)
fdr = fdrtool(pvalues, statistic="pvalue")
fdr$qval # estimated Fdr values 
fdr$lfdr # estimated local fdr 

# the same but with black and white figure  
fdr = fdrtool(pvalues, statistic="pvalue", color.figure=FALSE)


# estimate fdr and Fdr from z-scores

sd.true = 2.232
n = 500
z = rnorm(n, sd=sd.true)
z = c(z, runif(30, 5, 10)) # add some contamination
fdr = fdrtool(z)

# you may change some parameters of the underlying functions
fdr = fdrtool(z, cutoff.method="pct0", pct0=0.9) 
}
\keyword{htest}

\name{hc.score}
\alias{hc.score}
\alias{hc.thresh}

\title{Compute Empirical Higher Criticism Scores and Corresponding Decision Threshold From p-Values}

\usage{
hc.score(pval)
hc.thresh(pval, alpha0=1, plot=TRUE)
}

\description{
  \code{hc.score} computes the empirical higher criticism (HC) scores from p-values.

  \code{hc.thresh} determines the HC decision threshold by searching for the p-value with the maximum HC score.
}

\arguments{
  \item{pval}{vector of p-values.}
  \item{alpha0}{look only at a fraction \code{alpha0} of the p-values (default: 1, i.e. all p-values).}
  \item{plot}{show plot with HC decision threshold.}
}
\details{

   Higher Criticism (HC) provides an alternative means to determine decision thresholds for
   signal identification, especially if the signal is rare and weak.

   See Donoho and Jin (2008) for details of this approach 
   and Klaus and Strimmer (2012) for a review and connections with FDR methdology. 
}
\value{
  \code{hc.score} returns a vector with the HC score corresponding to each p-value. 

  \code{hc.thresh} returns the p-value corresponding to the maximum HC score.
}

\author{
 Bernd Klaus and  Korbinian Strimmer (\url{http://www.strimmerlab.org}).
}
\references{

Donoho, D. and J. Jin. (2008). Higher criticism thresholding: optimal feature selection
when useful features are rare and weak. Proc. Natl. Acad. Sci. USA 105:14790-15795.

Klaus, B., and K. Strimmer (2013). Signal identification for rare and
weak features: higher criticism or false discovery rates?
Biostatistics 14: 129-143. <DOI:10.1093/biostatistics/kxs030>
}

\seealso{\code{\link{fdrtool}}.}

\examples{
# load "fdrtool" library
library("fdrtool")

# some p-values
data(pvalues)

# compute HC scores
hc.score(pvalues)

# determine HC threshold
hc.thresh(pvalues)
}
\keyword{htest}

\name{halfnormal}
\alias{halfnormal}

\alias{dhalfnorm}
\alias{phalfnorm}
\alias{qhalfnorm}
\alias{rhalfnorm}
\alias{sd2theta}
\alias{theta2sd}


\title{The Half-Normal Distribution}
\description{
  Density, distribution function, quantile function and random
  generation for the half-normal distribution with parameter \code{theta}.
}
\usage{
dhalfnorm(x, theta=sqrt(pi/2), log = FALSE)
phalfnorm(q, theta=sqrt(pi/2), lower.tail = TRUE, log.p = FALSE)
qhalfnorm(p, theta=sqrt(pi/2), lower.tail = TRUE, log.p = FALSE)
rhalfnorm(n, theta=sqrt(pi/2))
sd2theta(sd)
theta2sd(theta)
}
\arguments{
  \item{x,q}{vector of quantiles.}
  \item{p}{vector of probabilities.}
  \item{n}{number of observations. If \code{length(n) > 1}, the length
    is taken to be the number required.}
  \item{theta}{parameter of half-normal distribution.}
  \item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}
  \item{lower.tail}{logical; if TRUE (default), probabilities are
    \eqn{P[X \le x]}{P[X <= x]}, otherwise, \eqn{P[X > x]}{P[X > x]}.}
  \item{sd}{standard deviation of the zero-mean normal distribution 
            that corresponds to the half-normal with parameter \code{theta}. }
}
\value{
  \code{dhalfnorm} gives the density,
  \code{phalfnorm} gives the distribution function,
  \code{qhalfnorm} gives the quantile function, and
  \code{rhalfnorm} generates random deviates.
  \code{sd2theta} computes a \code{theta} parameter. 
  \code{theta2sd} computes a \code{sd} parameter. 
}
\details{
 \code{x = abs(z)} follows a half-normal distribution with
  if \code{z} is a normal variate with zero mean. 
 The half-normal distribution has density
  \deqn{
    f(x) =
    \frac{2 \theta}{\pi} e^{-x^2 \theta^2/\pi}}{
    f(x) = 2*theta/pi e^-(x^2*theta^2/pi)
  }
  It has mean  \eqn{E(x) = \frac{1}{\theta}}{E(x) = 1/theta} and variance 
  \eqn{Var(x) = \frac{\pi-2}{2 \theta^2}}{Var(x) = (pi-2)/(2*theta^2)}.
  
 
  The parameter \eqn{\theta}{theta} is related to the
  standard deviation \eqn{\sigma}{sigma} of the corresponding 
  zero-mean normal distribution by the equation
  \eqn{\theta = \sqrt{\pi/2}/\sigma}{theta = sqrt(pi/2)/sigma}.

  If \eqn{\theta}{theta} is not specified in the above functions 
  it assumes the default values of \eqn{\sqrt{\pi/2}}{sqrt(pi/2)}, 
  corresponding to \eqn{\sigma=1}{sigma=1}.  
}
\seealso{
  \code{\link{Normal}}.
}

\examples{
# load "fdrtool" library
library("fdrtool")


## density of half-normal compared with a corresponding normal
par(mfrow=c(1,2))

sd.norm = 0.64
x  = seq(0, 5, 0.01)
x2 = seq(-5, 5, 0.01)
plot(x, dhalfnorm(x, sd2theta(sd.norm)), type="l", xlim=c(-5, 5), lwd=2,
   main="Probability Density", ylab="pdf(x)")
lines(x2, dnorm(x2, sd=sd.norm), col=8 )


plot(x, phalfnorm(x, sd2theta(sd.norm)), type="l", xlim=c(-5, 5),  lwd=2,
   main="Distribution Function", ylab="cdf(x)")
lines(x2, pnorm(x2, sd=sd.norm), col=8 )

legend("topleft", 
c("half-normal", "normal"), lwd=c(2,1),
col=c(1, 8), bty="n", cex=1.0)

par(mfrow=c(1,1))


## distribution function

integrate(dhalfnorm, 0, 1.4, theta = 1.234)
phalfnorm(1.4, theta = 1.234)

## quantile function
qhalfnorm(-1) # NaN
qhalfnorm(0)
qhalfnorm(.5)
qhalfnorm(1)
qhalfnorm(2) # NaN

## random numbers
theta = 0.72
hz = rhalfnorm(10000, theta)
hist(hz, freq=FALSE)
lines(x, dhalfnorm(x, theta))

mean(hz) 
1/theta  # theoretical mean

var(hz)
(pi-2)/(2*theta*theta) # theoretical variance


## relationship with two-sided normal p-values
z = rnorm(1000)

# two-sided p-values
pvl = 1- phalfnorm(abs(z))
pvl2 = 2 - 2*pnorm(abs(z)) 
sum(pvl-pvl2)^2 # equivalent
hist(pvl2, freq=FALSE)  # uniform distribution

# back to half-normal scores
hz = qhalfnorm(1-pvl)
hist(hz, freq=FALSE)
lines(x, dhalfnorm(x))
}
\keyword{distribution}

\name{gcmlcm}
\alias{gcmlcm}

\title{Greatest Convex Minorant and Least Concave Majorant}

\usage{
gcmlcm(x, y, type=c("gcm", "lcm"))
}

\description{
  \code{gcmlcm} computes the greatest convex minorant (GCM) or the
  least concave majorant (LCM) of a piece-wise linear function.
}

\arguments{
  \item{x, y}{coordinate vectors of the piece-wise linear function. Note
   that the x values need to be unique and be arranged in sorted order.}
  \item{type}{specifies whether to compute the greatest convex 
        minorant (\code{type="gcm"}, the default) or the
        least concave majorant (\code{type="lcm"}).}
}

\details{

 The GCM is obtained by isotonic regression of the raw slopes,
 whereas the LCM is obtained by antitonic regression.
 See Robertson et al. (1988).
}

\value{
  A list with the following entries:
  
  \item{x.knots}{the x values belonging to the knots of the LCM/GCM curve}
  \item{y.knots}{the corresponding y values}
  \item{slope.knots}{the slopes of the corresponding line segments}
}

\references{
 
 Robertson, T., F. T. Wright, and R. L. Dykstra. 1988.  Order restricted
 statistical inference. John Wiley and Sons.

}
\author{
  Korbinian Strimmer (\url{http://www.strimmerlab.org}).
}
\seealso{\code{\link{monoreg}}.}

\examples{
# load "fdrtool" library
library("fdrtool")

# generate some data
x = 1:20
y = rexp(20)
plot(x, y, type="l", lty=3, main="GCM (red) and LCM (blue)")
points(x, y)

# greatest convex minorant (red)
gg = gcmlcm(x,y)
lines(gg$x.knots, gg$y.knots, col=2, lwd=2)

# least concave majorant (blue)
ll = gcmlcm(x,y, type="lcm")
lines(ll$x.knots, ll$y.knots, col=4, lwd=2)

}
\keyword{smooth}
